@@ -20,7 +20,11 @@
 #include <assert.h>
 #include <errno.h>
 #include <stdbool.h>
+#ifdef _WIN32
+#include <winsock2.h>
+#else
 #include <netinet/tcp.h>
+#endif
 
 #include "raop.h"
 #include "netutils.h"
@@ -32,7 +36,20 @@
 #include "utils.h"
 #include "plist/plist.h"
 
+#ifdef _WIN32
+#define CAST (char *)
+/* are these keepalive settings for WIN32 correct? */
+/* (taken from https://github.com/wegank/ludimus)  */
+#define TCP_KEEPALIVE 3
+#define TCP_KEEPCNT 16
+#define TCP_KEEPIDLE TCP_KEEPALIVE
+#define TCP_KEEPINTVL 17
+#else
+#define CAST
+#endif
+
 #define SEC 1000000
+
 /* for MacOS, where SOL_TCP and TCP_KEEPIDLE are not defined */
 #if !defined(SOL_TCP) && defined(IPPROTO_TCP)
 #define SOL_TCP IPPROTO_TCP
@@ -225,7 +242,9 @@ raop_rtp_mirror_thread(void *arg)
             break;
         }
 
-        if (stream_fd == -1 && FD_ISSET(raop_rtp_mirror->mirror_data_sock, &rfds)) {
+        if (stream_fd == -1 &&
+	    (raop_rtp_mirror && raop_rtp_mirror->mirror_data_sock >= 0) &&
+            FD_ISSET(raop_rtp_mirror->mirror_data_sock, &rfds)) {
             struct sockaddr_storage saddr;
             socklen_t saddrlen;
             logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "raop_rtp_mirror accepting client");
@@ -240,25 +259,26 @@ raop_rtp_mirror_thread(void *arg)
             struct timeval tv;
             tv.tv_sec = 0;
             tv.tv_usec = 5000;
-            if (setsockopt(stream_fd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) < 0) {
+            if (setsockopt(stream_fd, SOL_SOCKET, SO_RCVTIMEO, CAST &tv, sizeof(tv)) < 0) {
                 logger_log(raop_rtp_mirror->logger, LOGGER_ERR, "raop_rtp_mirror could not set stream socket timeout %d %s", errno, strerror(errno));
                 break;
             }
+
             int option;
             option = 1;
-            if (setsockopt(stream_fd, SOL_SOCKET, SO_KEEPALIVE, &option, sizeof(option)) < 0) {
+            if (setsockopt(stream_fd, SOL_SOCKET, SO_KEEPALIVE, CAST &option, sizeof(option)) < 0) {
                 logger_log(raop_rtp_mirror->logger, LOGGER_WARNING, "raop_rtp_mirror could not set stream socket keepalive %d %s", errno, strerror(errno));
             }
             option = 60;
-            if (setsockopt(stream_fd, SOL_TCP, TCP_KEEPIDLE, &option, sizeof(option)) < 0) {
+            if (setsockopt(stream_fd, SOL_TCP, TCP_KEEPIDLE, CAST &option, sizeof(option)) < 0) {
                 logger_log(raop_rtp_mirror->logger, LOGGER_WARNING, "raop_rtp_mirror could not set stream socket keepalive time %d %s", errno, strerror(errno));
             }
             option = 10;
-            if (setsockopt(stream_fd, SOL_TCP, TCP_KEEPINTVL, &option, sizeof(option)) < 0) {
+            if (setsockopt(stream_fd, SOL_TCP, TCP_KEEPINTVL, CAST &option, sizeof(option)) < 0) {
                 logger_log(raop_rtp_mirror->logger, LOGGER_WARNING, "raop_rtp_mirror could not set stream socket keepalive interval %d %s", errno, strerror(errno));
             }
             option = 6;
-            if (setsockopt(stream_fd, SOL_TCP, TCP_KEEPCNT, &option, sizeof(option)) < 0) {
+            if (setsockopt(stream_fd, SOL_TCP, TCP_KEEPCNT, CAST &option, sizeof(option)) < 0) {
                 logger_log(raop_rtp_mirror->logger, LOGGER_WARNING, "raop_rtp_mirror could not set stream socket keepalive probes %d %s", errno, strerror(errno));
             }
             readstart = 0;
@@ -268,7 +288,8 @@ raop_rtp_mirror_thread(void *arg)
 
             // The first 128 bytes are some kind of header for the payload that follows
             while (payload == NULL && readstart < 128) {
-                ret = recv(stream_fd, packet + readstart, 128 - readstart, 0);
+                unsigned char* pos  = packet + readstart;
+                ret = recv(stream_fd, CAST pos, 128 - readstart, 0);
                 if (ret <= 0) break;
                 readstart = readstart + ret;
             }
@@ -299,7 +320,7 @@ raop_rtp_mirror_thread(void *arg)
              * be the only ones with packet[5] = 0x10, and almost always have packet[5] = 0x10,  *
              * but occasionally have packet[5] = 0x00.                                           */
 
-            /* unencrypted SPS/PPS packets have packet[4:7] = 0x01 0x00 0x01 0x16                *
+            /* unencrypted SPS/PPS packets have packet[4:7] = 0x01 0x00 (0x16 or 0x56) 0x01      *
              * they are followed by an encrypted packet with the same timestamp in packet[8:15]  */
 
             /* "streaming report" packages have packet[4:7] = 0x05 0x00 0x00 0x00, and have no    *
@@ -315,7 +336,8 @@ raop_rtp_mirror_thread(void *arg)
 
             while (readstart < payload_size) {
                 // Payload data
-                ret = recv(stream_fd, payload + readstart, payload_size - readstart, 0);
+                unsigned char *pos = payload + readstart;
+                ret = recv(stream_fd, CAST pos, payload_size - readstart, 0);
                 if (ret <= 0) break;
                 readstart = readstart + ret;
             }
@@ -414,7 +436,7 @@ raop_rtp_mirror_thread(void *arg)
 #endif
                 payload_decrypted = NULL;
                 h264_decode_struct h264_data;
-                h264_data.pts = ntp_timestamp;
+                h264_data.ntp_time = ntp_timestamp;
                 h264_data.nal_count = nalus_count;   /*nal_count will be the number of nal units in the packet */
                 h264_data.data_len = payload_size;
                 h264_data.data = payload_out;
@@ -431,6 +453,10 @@ raop_rtp_mirror_thread(void *arg)
             case 0x01:
                 // The information in the payload contains an SPS and a PPS NAL
                 // The sps_pps is not encrypted
+                if (payload_size == 0) {
+                    logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "raop_rtp_mirror, discard type 0x01 packet with no payload");
+                    break;
+                }
                 ntp_timestamp_nal = byteutils_get_long(packet, 8);
                 float width = byteutils_get_float(packet, 16);
                 float height = byteutils_get_float(packet, 20);
