@@ -16,6 +16,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
+#include <time.h>
+#include <stdint.h>
 
 char *
 utils_strsep(char **stringp, const char *delim)
@@ -196,6 +198,7 @@ char *utils_data_to_string(const unsigned char *data, int datalen, int chars_per
     sprintf(p,"\n");
     p++;
     assert(p == &(str[len]));
+    assert(len == strlen(str));
     return str;
 }
 
@@ -210,4 +213,23 @@ char *utils_data_to_text(const char *data, int datalen) {
     }
     return ptr;
 }
-  
+
+void ntp_timestamp_to_time(uint64_t ntp_timestamp, char *timestamp, size_t maxsize) {
+    time_t rawtime = (time_t) (ntp_timestamp / 1000000);
+    struct tm ts = *localtime(&rawtime);
+    assert(maxsize > 26);
+#ifdef _WIN32  /*modification for compiling for Windows */
+    strftime(timestamp, 20, "%Y-%m-%d %H:%M:%S", &ts);
+#else
+    strftime(timestamp, 20, "%F %T", &ts);
+#endif
+    snprintf(timestamp + 19, 8,".%6.6u", (unsigned int) ntp_timestamp % 1000000);
+}
+
+void ntp_timestamp_to_seconds(uint64_t ntp_timestamp, char *timestamp, size_t maxsize) {
+    time_t rawtime = (time_t) (ntp_timestamp / 1000000);
+    struct tm ts = *localtime(&rawtime);
+    assert(maxsize > 9);
+    strftime(timestamp, 3, "%S", &ts);
+    snprintf(timestamp + 2, 8,".%6.6u", (unsigned int) ntp_timestamp % 1000000);
+}
