@@ -15,6 +15,8 @@
 #ifndef UTILS_H
 #define UTILS_H
 
+#include <stdint.h>
+
 char *utils_strsep(char **stringp, const char *delim);
 int utils_read_file(char **dst, const char *pemstr);
 int utils_hwaddr_raop(char *str, int strlen, const char *hwaddr, int hwaddrlen);
@@ -22,4 +24,6 @@ int utils_hwaddr_airplay(char *str, int strlen, const char *hwaddr, int hwaddrle
 char *utils_parse_hex(const char *str, int str_len, int *data_len);
 char *utils_data_to_string(const unsigned char *data, int datalen, int chars_per_line);
 char *utils_data_to_text(const char *data, int datalen);
+void ntp_timestamp_to_time(uint64_t ntp_timestamp, char *timestamp, size_t maxsize);
+void ntp_timestamp_to_seconds(uint64_t ntp_timestamp, char *timestamp, size_t maxsize);
 #endif
