@@ -51,14 +51,15 @@ static gboolean check_plugins (void)
     gboolean ret;
     GstRegistry *registry;
     const gchar *needed[] = { "app", "libav", "playback", "autodetect", "videoparsersbad",  NULL};
-
+    const gchar *gst[] = {"plugins-base", "libav", "plugins-base", "plugins-good", "plugins-bad", NULL};
     registry = gst_registry_get ();
     ret = TRUE;
     for (i = 0; i < g_strv_length ((gchar **) needed); i++) {
         GstPlugin *plugin;
         plugin = gst_registry_find_plugin (registry, needed[i]);
         if (!plugin) {
-            g_print ("Required gstreamer plugin '%s' not found\n", needed[i]);
+            g_print ("Required gstreamer plugin '%s' not found\n"
+                     "Missing plugin is contained in  '[GStreamer 1.x]-%s'\n",needed[i], gst[i]);
             ret = FALSE;
             continue;
         }
@@ -68,41 +69,71 @@ static gboolean check_plugins (void)
     return ret;
 }
 
-void  gstreamer_init(){
-    gst_init(NULL,NULL);
-    g_assert(check_plugins ());
+bool gstreamer_init(uint64_t *unix_start_time, uint64_t *monotonic_start_time){
+    struct timespec tp;
+    GstClock *clock = NULL;
+    GstClockTime time;
+    
+    gst_init(NULL,NULL);    
+    clock = gst_system_clock_obtain();
+    if (!clock) {
+        g_print("gstreamer_init:  error: failed to obtain gst_system_clock\n");
+        return false;
+    }
+    g_object_set(clock, "clock-type", GST_CLOCK_TYPE_MONOTONIC, NULL);
+    time = GST_TIME_AS_NSECONDS(gst_clock_get_time(clock));
+    if (clock_gettime(CLOCK_REALTIME, &tp)){
+        g_print("gstreamer_init: error failed to get unix time\n");
+        return false;
+    }
+    time += GST_TIME_AS_NSECONDS(gst_clock_get_time(clock));
+    *monotonic_start_time = time/2;   
+    *unix_start_time = (1000000000 * tp.tv_sec) + tp.tv_nsec;
+    g_object_unref (clock);
+    return (bool) check_plugins ();
 }
 
 #define NFORMATS 2     /* set to 4 to enable AAC_LD and PCM:  allowed, but  never seen in real-world use */
 static audio_renderer_t *renderer_type[NFORMATS];
 static audio_renderer_t *renderer = NULL;
+static GstClockTime gst_audio_pipeline_base_time = GST_CLOCK_TIME_NONE;
 static logger_t *logger = NULL;
 const char * format[NFORMATS];
 
-void audio_renderer_init(logger_t *render_logger, const char* audiosink) {
+void audio_renderer_init(logger_t *render_logger, const char* audiosink, const char* audio_delay) {
     GError *error = NULL;
     GstCaps *caps = NULL;
+    GstClock *clock = gst_system_clock_obtain();
+    g_object_set(clock, "clock-type", GST_CLOCK_TYPE_MONOTONIC, NULL);
+
     logger = render_logger;
 
     for (int i = 0; i < NFORMATS ; i++) {
         renderer_type[i] = (audio_renderer_t *)  calloc(1,sizeof(audio_renderer_t));
         g_assert(renderer_type[i]);
         GString *launch = g_string_new("appsrc name=audio_source ! ");
-        g_string_append(launch, "queue ! ");
+        g_string_append(launch, "queue ");
         switch (i) {
         case 0:    /* AAC-ELD */
         case 2:    /* AAC-LC */
-            g_string_append(launch, "avdec_aac ! ");
+            g_string_append(launch, "! avdec_aac ! ");
             break;
         case 1:    /* ALAC */
-            g_string_append(launch, "avdec_alac ! ");
+            if (audio_delay[0]) {
+                g_string_append(launch, "min-threshold-time=");
+                g_string_append(launch, audio_delay);
+                g_string_append(launch, "000000 ");
+            }
+            g_string_append(launch, "! avdec_alac ! ");
             break;
         case 3:   /*PCM*/
             break;
         default:
             break;
         }
-        g_string_append (launch, "audioconvert ! volume name=volume ! level ! ");
+        g_string_append (launch, "audioconvert ! ");
+        g_string_append (launch, "audioresample ! ");    /* wasapisink must resample from 44.1 kHz to 48 kHz */
+        g_string_append (launch, "volume name=volume ! level ! ");
         g_string_append (launch, audiosink);
         g_string_append (launch, " sync=false");
         renderer_type[i]->pipeline  = gst_parse_launch(launch->str, &error);
@@ -110,9 +141,10 @@ void audio_renderer_init(logger_t *render_logger, const char* audiosink) {
           g_error ("gst_parse_launch error (audio %d):\n %s\n", i+1, error->message);
           g_clear_error (&error);
         }
-        g_string_free(launch, TRUE);
+
         g_assert (renderer_type[i]->pipeline);
- 
+        gst_pipeline_use_clock(GST_PIPELINE_CAST(renderer_type[i]->pipeline), clock);
+
         renderer_type[i]->appsrc = gst_bin_get_by_name (GST_BIN (renderer_type[i]->pipeline), "audio_source");
         renderer_type[i]->volume = gst_bin_get_by_name (GST_BIN (renderer_type[i]->pipeline), "volume");
         switch (i) {
@@ -139,9 +171,12 @@ void audio_renderer_init(logger_t *render_logger, const char* audiosink) {
         default:
             break;
         }
-        logger_log(logger, LOGGER_DEBUG, "supported audio format %d: %s",i+1,format[i]);
+        logger_log(logger, LOGGER_DEBUG, "Audio format %d: %s",i+1,format[i]);
+        logger_log(logger, LOGGER_DEBUG, "GStreamer audio pipeline %d: \"%s\"", i+1, launch->str);
+        g_string_free(launch, TRUE);
         g_object_set(renderer_type[i]->appsrc, "caps", caps, "stream-type", 0, "is-live", TRUE, "format", GST_FORMAT_TIME, NULL);
         gst_caps_unref(caps);
+        g_object_unref(clock);
     }
 }
 
@@ -169,21 +204,29 @@ void  audio_renderer_start(unsigned char *ct) {
             logger_log(logger, LOGGER_INFO, "changed audio connection, format %s", format[id]);
             renderer = renderer_type[id];
             gst_element_set_state (renderer->pipeline, GST_STATE_PLAYING);
+            gst_audio_pipeline_base_time = gst_element_get_base_time(renderer->appsrc);
         }
     } else if (compression_type) {
         logger_log(logger, LOGGER_INFO, "start audio connection, format %s", format[id]);
         renderer = renderer_type[id];
         gst_element_set_state (renderer->pipeline, GST_STATE_PLAYING);
+        gst_audio_pipeline_base_time = gst_element_get_base_time(renderer->appsrc);
     } else {
         logger_log(logger, LOGGER_ERR, "unknown audio compression type ct = %d", *ct);
     }
     
 }
 
-void audio_renderer_render_buffer(raop_ntp_t *ntp, unsigned char* data, int data_len, uint64_t ntp_time,
-                                  uint64_t rtp_time, unsigned short seqnum) {
+void audio_renderer_render_buffer(unsigned char* data, int *data_len, unsigned short *seqnum, uint64_t *pts_raw) {
     GstBuffer *buffer;
     bool valid;
+    GstClockTime pts = (GstClockTime) *pts_raw;
+    if (pts >= gst_audio_pipeline_base_time) {
+        pts -= gst_audio_pipeline_base_time;
+    } else {
+        logger_log(logger, LOGGER_ERR, "*** invalid *pts_raw < gst_audio_pipeline_base_time");
+        return;
+    }
     if (data_len == 0 || renderer == NULL) return;
 
     /* all audio received seems to be either ct = 8 (AAC_ELD 44100/2 spf 460 ) AirPlay Mirror protocol *
@@ -193,10 +236,10 @@ void audio_renderer_render_buffer(raop_ntp_t *ntp, unsigned char* data, int data
      *                   but is 0x80, 0x81 or 0x82: 0x100000(00,01,10) in ios9, ios10 devices          *
      * first byte of AAC_LC should be 0xff (ADTS) (but has never been  seen).                          */
     
-    buffer = gst_buffer_new_and_alloc(data_len);
+    buffer = gst_buffer_new_allocate(NULL, *data_len, NULL);
     g_assert(buffer != NULL);
-    GST_BUFFER_PTS(buffer) = (GstClockTime) ntp_time;
-    gst_buffer_fill(buffer, 0, data, data_len);
+    GST_BUFFER_PTS(buffer) = pts;
+    gst_buffer_fill(buffer, 0, data, *data_len);
     switch (renderer->ct){
     case 8: /*AAC-ELD*/
         switch (data[0]){
