@@ -22,7 +22,12 @@
 #include <gst/app/gstappsrc.h>
 
 #ifdef X_DISPLAY_FIX
+#include <gst/video/navigation.h>
 #include "x_display_fix.h"
+static bool fullscreen = false;
+static bool alt_keypress = false;
+#define MAX_X11_SEARCH_ATTEMPTS 5   /*should be less than 256 */
+static unsigned char X11_search_attempts; 
 #endif
 
 struct video_renderer_s {
@@ -92,17 +97,19 @@ static void append_videoflip (GString *launch, const videoflip_t *flip, const vi
 }	
 
 static video_renderer_t *renderer = NULL;
+static GstClockTime gst_video_pipeline_base_time = GST_CLOCK_TIME_NONE;
 static logger_t *logger = NULL;
 static unsigned short width, height, width_source, height_source;  /* not currently used */
 static bool first_packet = false;
 
-/* apple uses colorimetry=1:3:5:1 (not recognized by gstreamer v4l2)  *
+/* apple uses colorimetry=1:3:5:1                                *
+ * (not recognized by v4l2 plugin in Gstreamer  < 1.20.4)        *
  * See .../gst-libs/gst/video/video-color.h in gst-plugins-base  *
  * range = 1   -> GST_VIDEO_COLOR_RANGE_0_255      ("full RGB")  * 
  * matrix = 3  -> GST_VIDEO_COLOR_MATRIX_BT709                   *
  * transfer = 5 -> GST_VIDEO_TRANSFER_BT709                      *
  * primaries = 1 -> GST_VIDEO_COLOR_PRIMARIES_BT709              *
- * closest is BT709, 2:3:5:1 with                                *
+ * closest used by  GStreamer < 1.20.4 is BT709, 2:3:5:1 with    *                            *
  * range = 2 -> GST_VIDEO_COLOR_RANGE_16_235 ("limited RGB")     */  
 
 static const char h264_caps[]="video/x-h264,stream-format=(string)byte-stream,alignment=(string)au";
@@ -116,9 +123,12 @@ void video_renderer_size(float *f_width_source, float *f_height_source, float *f
 }
 
 void  video_renderer_init(logger_t *render_logger, const char *server_name, videoflip_t videoflip[2], const char *parser,
-                          const char *decoder, const char *converter, const char *videosink) {
+                          const char *decoder, const char *converter, const char *videosink, const bool *initial_fullscreen) {
     GError *error = NULL;
     GstCaps *caps = NULL;
+    GstClock *clock = gst_system_clock_obtain();
+    g_object_set(clock, "clock-type", GST_CLOCK_TYPE_MONOTONIC, NULL);
+
     logger = render_logger;
 
     /* this call to g_set_application_name makes server_name appear in the  X11 display window title bar, */
@@ -149,30 +159,40 @@ void  video_renderer_init(logger_t *render_logger, const char *server_name, vide
         g_clear_error (&error);
     }
     g_assert (renderer->pipeline);
-    g_string_free(launch, TRUE);
+    gst_pipeline_use_clock(GST_PIPELINE_CAST(renderer->pipeline), clock);
 
     renderer->appsrc = gst_bin_get_by_name (GST_BIN (renderer->pipeline), "video_source");
     g_assert(renderer->appsrc);
     caps = gst_caps_from_string(h264_caps);
     g_object_set(renderer->appsrc, "caps", caps, "stream-type", 0, "is-live", TRUE, "format", GST_FORMAT_TIME, NULL);
+    g_string_free(launch, TRUE);
     gst_caps_unref(caps);
+    gst_object_unref(clock);
 
     renderer->sink = gst_bin_get_by_name (GST_BIN (renderer->pipeline), "video_sink");
     g_assert(renderer->sink);
 
 #ifdef X_DISPLAY_FIX
+    fullscreen = *initial_fullscreen;
     renderer->server_name = server_name;
     renderer->gst_window = NULL;
     bool x_display_fix = false;
-    if (strcmp(videosink,"autovideosink") == 0 ||
-        strcmp(videosink,"ximagesink") ==  0 ||
-        strcmp(videosink,"xvimagesink") == 0) {
+    /* only include X11 videosinks that provide fullscreen mode, or need ZOOMFIX */
+    /* limit searching for X11 Windows in case autovideosink selects an incompatible videosink */
+    if (strncmp(videosink,"autovideosink", strlen("autovideosink")) == 0 ||
+        strncmp(videosink,"ximagesink", strlen("ximagesink")) ==  0 ||
+	strncmp(videosink,"xvimagesink", strlen("xvimagesink")) == 0 ||
+	strncmp(videosink,"fpsdisplaysink", strlen("fpsdisplaysink")) == 0 ) {
         x_display_fix = true;
     }
     if (x_display_fix) {
         renderer->gst_window = calloc(1, sizeof(X11_Window_t));
         g_assert(renderer->gst_window);
         get_X11_Display(renderer->gst_window);
+        if (!renderer->gst_window->display) {
+            free(renderer->gst_window);
+            renderer->gst_window = NULL;
+        }
     }
 #endif
     gst_element_set_state (renderer->pipeline, GST_STATE_READY);
@@ -190,12 +210,23 @@ void  video_renderer_init(logger_t *render_logger, const char *server_name, vide
 
 void video_renderer_start() {
     gst_element_set_state (renderer->pipeline, GST_STATE_PLAYING);
+    gst_video_pipeline_base_time = gst_element_get_base_time(renderer->appsrc);
     renderer->bus = gst_element_get_bus(renderer->pipeline);
     first_packet = true;
+#ifdef X_DISPLAY_FIX
+    X11_search_attempts = 0;
+#endif
 }
 
-void video_renderer_render_buffer(raop_ntp_t *ntp, unsigned char* data, int data_len, uint64_t pts, int nal_count) {
+void video_renderer_render_buffer(unsigned char* data, int *data_len, int *nal_count, uint64_t *pts_raw) {
     GstBuffer *buffer;
+    GstClockTime pts = (GstClockTime) *pts_raw;
+    if (pts >= gst_video_pipeline_base_time) {
+        pts -= gst_video_pipeline_base_time;
+    } else {
+        logger_log(logger, LOGGER_ERR, "*** invalid *pts_raw < gst_video_pipeline_base_time") ;
+        return;
+    }
     g_assert(data_len != 0);
     /* first four bytes of valid  h264  video data are 0x00, 0x00, 0x00, 0x01.    *
      * nal_count is the number of NAL units in the data: short SPS, PPS, SEI NALs *
@@ -208,14 +239,24 @@ void video_renderer_render_buffer(raop_ntp_t *ntp, unsigned char* data, int data
             logger_log(logger, LOGGER_INFO, "Begin streaming to GStreamer video pipeline");
             first_packet = false;
         }
-        buffer = gst_buffer_new_and_alloc(data_len);
+        buffer = gst_buffer_new_allocate(NULL, *data_len, NULL);
         g_assert(buffer != NULL);
-        GST_BUFFER_PTS(buffer) = (GstClockTime) pts;
-        gst_buffer_fill(buffer, 0, data, data_len);
+        GST_BUFFER_PTS(buffer) = pts;
+        gst_buffer_fill(buffer, 0, data, *data_len);
         gst_app_src_push_buffer (GST_APP_SRC(renderer->appsrc), buffer);
 #ifdef X_DISPLAY_FIX
-        if (renderer->gst_window && !(renderer->gst_window->window)) {
-            fix_x_window_name(renderer->gst_window, renderer->server_name);
+        if (renderer->gst_window && !(renderer->gst_window->window) && X11_search_attempts < MAX_X11_SEARCH_ATTEMPTS) {
+            X11_search_attempts++;
+            logger_log(logger, LOGGER_DEBUG, "Looking for X11 UxPlay Window, attempt %d", (int) X11_search_attempts);
+            get_x_window(renderer->gst_window, renderer->server_name);
+	    if (renderer->gst_window->window) {
+                logger_log(logger, LOGGER_INFO, "\n*** X11 Windows: Use key F11 or (left Alt)+Enter to toggle full-screen mode\n");
+                if (fullscreen) {
+                    set_fullscreen(renderer->gst_window, &fullscreen);
+                }
+            } else if (X11_search_attempts == MAX_X11_SEARCH_ATTEMPTS) {
+	      logger_log(logger, LOGGER_DEBUG, "X11 UxPlay Window not found in %d search attempts", MAX_X11_SEARCH_ATTEMPTS);
+            }
         }
 #endif
     }
@@ -289,6 +330,43 @@ gboolean gstreamer_pipeline_bus_callback(GstBus *bus, GstMessage *message, gpoin
          logger_log(logger, LOGGER_INFO, "GStreamer: End-Of-Stream");
 	//   g_main_loop_quit( (GMainLoop *) loop);
         break;
+#ifdef  X_DISPLAY_FIX
+    case GST_MESSAGE_ELEMENT:
+        if (renderer->gst_window && renderer->gst_window->window) {
+            GstNavigationMessageType message_type = gst_navigation_message_get_type (message);
+            if (message_type == GST_NAVIGATION_MESSAGE_EVENT) {
+                GstEvent *event = NULL;
+                if (gst_navigation_message_parse_event (message, &event)) {
+                    GstNavigationEventType event_type = gst_navigation_event_get_type (event);
+                    const gchar *key;
+                    switch (event_type) {
+                    case GST_NAVIGATION_EVENT_KEY_PRESS:
+                        if (gst_navigation_event_parse_key_event (event, &key)) {
+                            if ((strcmp (key, "F11") == 0) || (alt_keypress && strcmp (key, "Return") == 0)) {
+                                fullscreen = !(fullscreen);
+                                set_fullscreen(renderer->gst_window, &fullscreen);
+                            } else if (strcmp (key, "Alt_L") == 0) {
+                                alt_keypress = true;
+                            }
+                        }
+                        break;
+                    case GST_NAVIGATION_EVENT_KEY_RELEASE:
+                        if (gst_navigation_event_parse_key_event (event, &key)) {
+                            if (strcmp (key, "Alt_L") == 0) {
+                                alt_keypress = false;
+                            }
+                        }
+                    default:
+                        break;
+                    }
+                }
+                if (event) {
+                    gst_event_unref (event);
+                }
+            }
+        }
+        break;
+#endif
     default:
       /* unhandled message */
         break;
